/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __CELL_CPP
#define __CELL_CPP

#include "cell.h"
#include "patch.h"
#include "individual.h"
#include "behaviour.h"
#include "logging.h"

Cell::Cell()
{
	cover_id = 0;
	disturbance = NULL_TYPE;
	used_resources = 0;
	patch = NULL;
}

Cell::Cell(cover_type _cover_id, disturbance_type _disturbance, Patch* _patch)
{
	cover_id = _cover_id;
	disturbance = _disturbance;
	used_resources = 0;
	patch = _patch;
}

Cell::~Cell()
{
}

void Cell::reset()
{
	cover_id = 0;
	disturbance = NULL_TYPE;
	used_resources = 0;
	patch = NULL;
}

void Cell::use_Resources(Individual* _individual)
{
	_individual->setCell(this);
	used_resources+=_individual->getBehaviour()->getResource_factor();
}

void Cell::release_Resources(Individual* _individual)
{
	if(used_resources < 0)
		Logging::log_error("Cell::release_Resources() used_resources < 0!");
	used_resources-=_individual->getBehaviour()->getResource_factor();
}

#endif