/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __CELL_H
#define __CELL_H

#include "defs.h"

class Patch;
class Individual;

/**
	A cell corresponds to one pixel in a square grid. Cells are 
	arranged in a two dimensional array. Each cell contains information
	related to its virtual space. A Cell is not aware of its spatial 
	location. A cell is part of a Patch, which in turn is part of a 
	Landscape. Patches are composed of Cells. Landscapes are composed 
	of Patches - and Cells.
*/

class Cell 
{
private:
	cover_type			cover_id;			//id of the landcover type (0..255)
    disturbance_type	disturbance;		//disturbance type (0..255) - not used in Version 1.0
    float				used_resources;		//ressource counter - for carrying capacity
    Patch*				patch;				//Patch to which this Cell belongs to

public:
	Cell();
	Cell(cover_type _cover_id, disturbance_type _disturbance, Patch* _patch);
	~Cell();
	void reset();

    cover_type getCover_id() const						{ return cover_id; };
	void setCover_id(cover_type _cover_id)				{ cover_id = _cover_id; };

    disturbance_type getDisturbance() const				{ return disturbance; };
	void setDisturbance(disturbance_type _disturbance)	{ disturbance = _disturbance; };	
											
	Patch* getPatch() const								{ return patch; };
	void setPatch(Patch* _patch)						{ patch = _patch; };

	float getUsed_resources() const						{ return used_resources; };

	void use_Resources(Individual* _individual);		
	void release_Resources(Individual* _individual);	
};

#endif