/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __DEFS_H
#define __DEFS_H

#include <limits.h>
#include <float.h>

//type definitions for state variables
typedef unsigned char cover_type;
typedef unsigned char age_type;
typedef unsigned char disturbance_type;
typedef unsigned char allele_type;

//invalid value for all state variables
#define NULL_TYPE UCHAR_MAX
#define NULL_INT INT_MIN
#define NULL_FLOAT FLT_MIN
#define NULL_LONG LONG_MIN
#define NULL_SHORT USHRT_MAX

#define NULL 0
#define PIE 3.1415927
//resolution for tracing individual movement steps
#define TRACE_RESOLUTION 0.5

//definition of possible border types
enum border_types {TORUS, ABSORB, REFLECT, OPEN};
//definition of landcover source origins
enum sources { NOT_DEFINED, IMPORTED, GENERATED };
//status of the landscape model
enum status_types { NOT_INITIALIZED, CELL_TOPOLOGY, PATCH_TOPOLOGY, PATCH_ISOL_METRICS};
//colors for gif file generation
enum gifcolors { WHITE, LIGHTGREEN, DARKGREEN, YELLOW, GREY, BLUE, RED, BLACK };
//genders of individuals
enum gender_types { NOTHING, MALE, FEMALE };
//initial distribution types for population
enum combination_types { P_FACTORIAL, P_RANDOM };
//initial distribution types for population
enum distribution_types { INVALID, LANDSCAPE, HABITAT, MATRIX, COVER_TYPE, PATCH, CELL };
//activity sequence or order
enum sequence_types { DEFAULT, RANDOM, CUSTOM };
//individual activities
enum activity_types { REPRODUCTION, MOVEMENT, MORTALITY, MATURE };
//data types
enum data_types { _FLOAT, _INTEGER, _STRING};

//turn on/off logging
#define LOG_ERROR 1
//#define LOG_DEBUG 1
#define LOG_INFO 1
#define LOG_WARNING 1
#define LOG_FILE 1
#define LOG_SCREEN 1

//maximum grid extensions
#define XMAX 1000
#define YMAX 1000

//maximum population sizes
#define PMAX 100000

#endif