/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __DYNINTPAIR_H
#define __DYNINTPAIR_H

//diparirlist is a dynamic list of integer pairs, like
//coordinate pairs 

#include <iostream>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif
 
struct intpair_el
{
    int xval, yval;
    intpair_el* right;
};
 
//one dimensional dynamic array of integer pairs
 
class dipairlist
{
    intpair_el* leftpair;
    intpair_el* rightpair;
public:  
	dipairlist();
	//creates dipairlist, initialize with 0
	dipairlist(int x);
	//copy constructor
	dipairlist(dipairlist& ilist);
	~dipairlist();
	//delete intpair_els
	void reset();
 
	void add(int x, int y);
	int operator=(dipairlist& ilist);
	int operator==(dipairlist& ilist);
	int operator!=(dipairlist& ilist);
	int entries() const;
	int index_of(int x, int y) const;
	int get_xval_at(int i) const;
	int get_yval_at(int i) const;
	void report(ostream& strm=cout) const;
private:
	void error(int n, float val=0) const;
};

#endif

