/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __DYNVOIDARRAY_H
#define __DYNVOIDARRAY_H

//dvoidlist is a dynamic list of pointers to void, dynamic
//means, you can add single list_elements, listsize is not to
//be defined somewhere
//dvoidarr is a dynamic array of pointer to void, dynamic means
//you can add a dvoidlist whith any size or empty any time as
//long as memory is available, however dvoidlist's (rows) cannot be 
//changed in size later on, just the number of columns is 

#include <iostream>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

struct void_el
{
	void* val;
	void_el* right;
};

//one dimensional dynamic array of pointers to void

class dvoidlist
{
	void_el* leftvoid;
	void_el* rightvoid;
public:
	dvoidlist();
	//creates list, initialize with 0
	dvoidlist(int x, int* dummy);
	~dvoidlist();
	//delete void_els
	void reset();
	
	void add(void* v);
	void* operator[](int x);
	int set(int x, void* v);
	void* get(int x) const;
	int entries() const;
	int elem_index(void* v) const;
	void report(ostream& strm=cout) const;
private:
	void error(int n, float val=0) const;
};

struct voidlist_el
{
	dvoidlist* lval;
	voidlist_el* right;
};

class dvoidarray
{
	voidlist_el* toplist;
	voidlist_el* bottomlist;	
public:
	dvoidarray();
	//creates array, initialize with 0
	dvoidarray(int x, int y);
	~dvoidarray();
	void reset();

	int add(dvoidlist* il);
	dvoidlist& operator[](int y);
	int set(int x, int y, void* p);
	void* get(int x, int y) const;
	int entries() const;
	void report(ostream& strm=cout) const;
private:
	void error(int n, float val=0) const;
};

#endif
