/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __GIF_CPP 
#define __GIF_CPP

#include <iostream>
#include <stdlib.h>
#include "gif.h"
#include "landscape.h"


Gifsaver::Gifsaver(Landscape* _landscape)
{
	landscape = _landscape;
}

Gifsaver::~Gifsaver()
{
}

int Gifsaver::landscape_colors(int x, int y)
{
	cover_type cover_id = grid[x][y].getCover_id();
	return cover_types[cover_id]->getColor();
}

int Gifsaver::save_landscape(char* filename)
{
	if(!landscape)
	{
		error(1);
		return 0;
	}
 
	int xdim=landscape->getXdim();
	int ydim=landscape->getYdim();
	int err;
  	set_landscape_colors();
  	err=GIF_Create(filename,xdim,ydim,lsc.numcolors, lsc.colorres);
  	if(err!=GIF_OK) 
		GIF_exit(err);
   	for(int i=0;i<lsc.numcolors;i++)
     		GIF_SetColor(i,lsc.red[i],lsc.green[i],lsc.blue[i]);
   	err=GIF_CompressImage(0,0,-1,-1, &landscape_colors);
   	if(err!=GIF_OK) 
		GIF_exit(err);
   	err=GIF_Close();
   	if(err!=GIF_OK) 
		GIF_exit(err);
  	del_lsccolors();
  	return 1;
}

void Gifsaver::set_landscape_colors()
{
  	lsc.red=new int[8];
  	lsc.green=new int[8];
  	lsc.blue=new int[8];
  	if(!lsc.red || !lsc.green || !lsc.blue )
	{
    	error(2);
		exit(-1);
    }
  	lsc.numcolors=8;
  	lsc.colorres=4;
  	lsc.red[0]=15;	lsc.green[0]=15;	lsc.blue[0]=15;		// 0=white (error)
  	lsc.red[1]=4;	lsc.green[1]=10;  	lsc.blue[1]=0;		// 1=light green (HOSP)
  	lsc.red[2]=0;	lsc.green[2]=6;  	lsc.blue[2]=0;		// 2=dark green (BREEDING)
  	lsc.red[3]=12;	lsc.green[3]=12;   	lsc.blue[3]=0;		// 3=yellow (INHOSP)
  	lsc.red[4]=13;	lsc.green[4]=13;   	lsc.blue[4]=13;		// 4=grey
  	lsc.red[5]=0;	lsc.green[5]=0;   	lsc.blue[5]=15;		// 5=blue
  	lsc.red[6]=15;	lsc.green[6]=0;   	lsc.blue[6]=0;		// 6=red
  	lsc.red[7]=0;	lsc.green[7]=0;   	lsc.blue[7]=0;		// 7=black (rahmen)
}


void Gifsaver::del_lsccolors()
{
    if(lsc.red) 
        delete lsc.red;
    if(lsc.green)
  	    delete lsc.green;
    if(lsc.blue)
         delete lsc.blue;
    lsc.numcolors=0;
    lsc.colorres=0;
}


void Gifsaver::error(int n) const
{
	cout << "ERROR CLASS gif_saver::";
    switch(n) 
	{
        case 1: cout << "save_landscape(): landscape not created!" << endl;
                break;
        case 2: cout << "set_landscape_colors(): lack of memory!" << endl;
                break;
		default: cout << "error(): no error et all?" << endl;
    }
};


#endif 

