/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __UTIL_CPP
#define __UTIL_CPP

#include "statisti.h"
#include "ibusutil.h"
#include "logging.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

long Utility::ltime = time( &ltime );
struct tm Utility::d = *localtime( &ltime );
char* Utility::buffer = new char[30];

void Utility::write_date(ostream& strm)
{
   time( &ltime );
   d = *localtime( &ltime );
   
   char month[10];
   char year[10];
   char day[10];
   char hour[10];
   char min[10];
   char sec[10];
   sprintf(month,"%02d",++d.tm_mon);
   sprintf(year,"%02d",d.tm_year+1900);
   sprintf(day,"%02d",d.tm_mday);
   sprintf(hour,"%02d",d.tm_hour);
   sprintf(min,"%02d",d.tm_min);
   sprintf(sec,"%02d",d.tm_sec);
   
   strm<<"["<<month<<"/"<<day<<"/"<<year<<" "<<hour<<":"<<min<<":"<<sec<<"] ";
}

int Utility::randint(int range)
{
	int r = rand();
	int rand = (r == RAND_MAX ? r-1 : r);
	return rand * range / RAND_MAX;
}

float Utility::randint(int from, int to)
{
	if(from < to)
		return (rand() * (to - from + 1 ) / RAND_MAX) + from;
	return (rand() * (from - to) / RAND_MAX) + to;
}

float Utility::randfloat(float range)
{
	return (float)rand() * range / (float)RAND_MAX;
}

float Utility::randfloat(float from, float to)
{
	if(from < to)
		return ((float)rand() * (to - from) / (float)RAND_MAX) + from;
	return ((float)rand() * (from - to) / (float)RAND_MAX) + to;
}

char* Utility::int_to_char(int _val)
{
	sprintf(buffer,"%d",_val);
	return buffer;
}

char* Utility::float_to_char(float _val, float _prec)
{
	if(_prec == 0)
		sprintf(buffer,"%f",_val);
	else
		sprintf(buffer,"%0.2f",_val);
	return buffer;
}

int Utility::round(float _val)
{
	return (int)(_val + 0.5);
}

rdist* Utility::distribution_for_name(char* _distribution)
{
	if(!_distribution)
		return NULL;

	char distribution[100];
	sprintf(distribution,"%s",_distribution);

	char* name  = strtok(distribution,";");
	char* min_mean = strtok(NULL,";");
				
	if(!strcmp(name,"negexp") && min_mean)
		return new negexp("negexp",1/atof(min_mean));

	char* max = strtok(NULL,";");

	if(!strcmp(name,"normal") && min_mean && max)
		return new normal("normal",atof(min_mean), atof(max));

	if(!strcmp(name,"uniform") && min_mean && max)
		return new uniform("uniform",atof(min_mean), atof(max));

	return NULL;
}

poisson* Utility::poisson_for_name(char* _distribution)
{
	if(!_distribution)
		return NULL;

	char distribution[100];
	sprintf(distribution,"%s",_distribution);

	char* name  = strtok(distribution,";");
	char* min_mean = strtok(NULL,";");
				
	if(!strcmp(name,"poisson") && min_mean)
		return new poisson("poisson",atof(min_mean));

	return NULL;
}

sequence_types Utility::sequence_for_name(char* _sequence)
{
	if(!strcmp(_sequence,"default") || !strcmp(_sequence, "DEFAULT"))
		return DEFAULT;
	if(!strcmp(_sequence,"custom") || !strcmp(_sequence, "CUSTOM"))
		return CUSTOM;
	if(!strcmp(_sequence,"random") || !strcmp(_sequence, "RANDOM"))
		return RANDOM;
	return RANDOM;
}

activity_types Utility::activity_for_name(char* _activity)
{
	if(!strcmp(_activity,"death") || !strcmp(_activity,"MORTALITY"))
		return MORTALITY;
	if(!strcmp(_activity,"repro") || !strcmp(_activity,"REPRODUCTION"))
		return REPRODUCTION;
	if(!strcmp(_activity,"move") || !strcmp(_activity,"MOVEMENT"))
		return MOVEMENT;
	if(!strcmp(_activity,"aging") || !strcmp(_activity,"AGING"))
		return MATURE;
	Logging::log_warning("Utility::activity_for_name() could not recognize '",_activity,"'.\n",NULL);
	return MOVEMENT;
}

char* Utility::name_for_activity(activity_types _activity)
{
	switch(_activity)
	{
		case MORTALITY:		return "death";
		case REPRODUCTION:	return "repro";
		case MOVEMENT:		return "move";
		case MATURE:		return "age";
		default:			return "unknown";
	}
}

distribution_types Utility::individual_distribution_for_name(char* _distribution)
{
	if(!strcmp(_distribution,"landscape") || !strcmp(_distribution,"LANDSCAPE"))
		return LANDSCAPE;
	if(!strcmp(_distribution,"habitat") || !strcmp(_distribution,"HABITAT"))
		return HABITAT;
	if(!strcmp(_distribution,"matrix") || !strcmp(_distribution,"MATRIX"))
		return MATRIX;
	if(!strcmp(_distribution,"landcover") || !strcmp(_distribution,"LANDCOVER"))
		return COVER_TYPE;
	if(!strcmp(_distribution,"cell") || !strcmp(_distribution,"CELL"))
		return CELL;
	if(!strcmp(_distribution,"patch") || !strcmp(_distribution,"PATCH"))
		return PATCH;
	return INVALID;
}


#endif