/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __UTIL_H
#define __UTIL_H

#include <iostream>
#include "defs.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class rdist;
class poisson;

/**
	Class Utility provides utility functions to the model, such
	as transforming numerical values to alphanumerical values
	and random numbers. Utility is a stateless class. All methods
	are static.
*/

class Utility
{
private:
	static struct tm d;	
	static long ltime;
	static char* buffer;

public:
	Utility() {};
	~Utility() {};

	/* writes formatted date to ouput stream */
	static void write_date(ostream& strm=cout);

	/* generates a random number between from and to (from and to are included) */
	static float Utility::randint(int from, int to);
	
	/* generates a random number between 0 and range (range is included) */
	static int randint(int range=100);

	/* generates a random number between from and to (from and to are included) */
	static float randfloat(float from, float to);

	/* generates a floating point number between 0 and range (range is included) */
	static float randfloat(float range=1);

	/* transforms an integer into a const char* */
	static char* int_to_char(int _val);

	/* transforms a float into a const char* */
	static char* float_to_char(float _val, float _prec=0);

	/* rounds a float value into an integer */
	static int round(float _val);

	/* instantiates a distribution of type rdist based on a parameter string */
	static rdist* distribution_for_name(char* _distribution);
	
	/* instantiates a distribution of type poisson based on a parameter string */
	static poisson* Utility::poisson_for_name(char* _distribution);

	/* returns a sequence based on name */
	static sequence_types sequence_for_name(char* _sequence);

	/* returns a activity based on name */
	static activity_types activity_for_name(char* _activity);

	/* returns names for activities */
	static char* Utility::name_for_activity(activity_types _activity);
	
	/* returns individual distribution type based on name */
	static distribution_types individual_distribution_for_name(char* _distribution);
 };
 
 #endif