/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __LANDCOVERTYPE_CPP
#define __LANDCOVERTYPE_CPP

#include "landcovertype.h"

const cover_type LandCoverType::habitat = 1;
const cover_type LandCoverType::matrix  = 0;

LandCoverType::LandCoverType()
{
	cover_id = NULL_TYPE;
	cover_name = NULL;
	fragmentation = NULL_FLOAT;
	coverage = NULL_INT;
	sequence = NULL_INT;
	generated = false;
	gifcolor = WHITE;
	cell_reduction = 1;
	cell_capacity = NULL_FLOAT;
}
	
LandCoverType::LandCoverType(cover_type _cover_id, char* _cover_name, 
							 float _fragmentation, int _coverage, int _sequence,
							 gifcolors _gifcolor)
{
	cover_id = _cover_id;
	cover_name = _cover_name;
	fragmentation = _fragmentation;
	coverage = _coverage;
	sequence = _sequence;
	generated = false;
	gifcolor = _gifcolor;
	cell_reduction = 1;
	cell_capacity = NULL_FLOAT;
}

void LandCoverType::reset()
{
	fragmentation = NULL_FLOAT;
	coverage = NULL_INT;
	sequence = NULL_INT;
	generated = false;
	cell_reduction = 1;
	cell_capacity = NULL_FLOAT;
}

LandCoverType::~LandCoverType()
{
}

bool LandCoverType::operator == (const LandCoverType& _landcovertype) const
{
	return	_landcovertype.cover_id == cover_id &&
			_landcovertype.cover_name == cover_name;
}

bool LandCoverType::operator != (const LandCoverType& _landcovertype) const
{
	return !((*this)==_landcovertype);
}

void LandCoverType::report(ostream& strm) const
{
	strm<<"cover_id: "<<(int)cover_id<<" name: "<<cover_name<<" frag: "<<fragmentation;
	strm<<" cover: "<<coverage<<" seq: "<<sequence<<" color: "<<gifcolor;
	strm<<" cell_red: "<<cell_reduction<<" cell_cap: "<<cell_capacity<<"\n";
}

#endif