/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __LANDCOVERTYPE_H
#define __LANDCOVERTYPE_H

#include "defs.h"
#include <iostream>

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

/**
	A LandCoverType combines all information for a landcover type in a Landscape.
	LandCoverType objects are stored in a list and managed by the Landscape
	object. Each Cell identifies the corresponding LandCoverType by the cover_id.
*/

class LandCoverType 
{
private:
    cover_type		cover_id;		//landcover type (0..255)
    char*			cover_name;		//name of landcover type
    float			fragmentation;	//value for cox-pattern generation
	float			coverage;		//coverage (proportion)
	int				sequence;		//order of assignment (only when generated)	
	float			cell_capacity;	//resource capacity for each cell with this cover type
	float			cell_reduction;	//fraction of capacity per cell, left after removing individuals
									//due to overabundance, i.e. cell capacity exceeding 
	bool			generated;		//true, if cover type is generated on the landscape
	gifcolors		gifcolor;		//for gif file generation

public:
	static const cover_type habitat;
	static const cover_type matrix;

	LandCoverType();
	LandCoverType(cover_type _cover_id, char* _cover_name, float _fragmentation, 
				  int _coverage, int _sequence, gifcolors _gifcolor=WHITE);
	~LandCoverType();
	void reset();

    cover_type getCover_id() const						{ return cover_id; }
	void setCover_id(cover_type _cover_id)				{ cover_id = _cover_id; };

    char* getCover_name() const							{ return cover_name; };
	void setCover_name(char* _cover_name)				{ cover_name = _cover_name; };	
											
    float getFragmentation() const						{ return fragmentation; }
	void setFragmentation(float _fragmentation)			{ fragmentation = _fragmentation; };

	float getCoverage() const							{ return coverage; }
	void setCoverage(float _coverage)					{ coverage = _coverage; };

	int getSequence() const								{ return sequence; };
	void setSequence(int _sequence)						{ sequence = _sequence; };

	void setGenerated(bool _generated)					{ generated = _generated; };
	bool isGenerated() const							{ return generated; };

	void setColor(gifcolors _gifcolor)					{ gifcolor = _gifcolor; };
	gifcolors getColor() const							{ return gifcolor; };

	void setCell_reduction(float _cell_reduction)		{ cell_reduction = _cell_reduction; };
	float getCell_reduction() const						{ return cell_reduction; };

	float getCell_capacity() const						{ return cell_capacity; };
	void setCell_capacity(float _cell_capacity)			{ cell_capacity = _cell_capacity; };

	bool operator == (const LandCoverType& _landcovertype) const;
	bool operator != (const LandCoverType& _landcovertype) const;

	void report(ostream& strm=cout) const;
};

#endif