/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __LOGGING_H
#define __LOGGING_H

#include <iostream>
#include <fstream>
#include "defs.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

/**
	Logging allows to write log information on the screen or
	in log files. Logging also rotates log files. Log levels
	can be set in defs.h.
*/

class Logging
{

#ifdef LOG_FILE
	static const char* destination;	
	static ofstream* logfilestream;
#endif

public:
	Logging() {};
	~Logging() {};

	static void set_destination(const char* _destination);
	static void close_destination();

	static void log_error(const char* m1, ...);
	static void log_warning(const char* m1, ...);
	static void log_info(bool _header, const char* m1, ...);
	static void log_debug(const char* m1, ...);
};
 
#endif

