/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __MODELCONTROLLER_H
#define __MODELCONTROLLER_H

#include "modelregistry.h"
#include <time.h>

/**
	ModelController governs the principal execution of the model,
	such as reading parameter file, initialization of data structures,
	simulation, data aggregation and writing of results.
*/

class ModelController
{
private:
	ModelRegistry registry;
	int argcount;
	char** arguments;

	time_t	starttime;		//start time of simulation
	time_t	endtime;	 	//end time of simulation
	double	simtime;		//duration (real time) of simulation

public: 
	ModelController(int _argc, char** _argv);
	~ModelController() {};
	
	void execute();

private:
	void randomize();
	void read_parameters() throw (IbusepmException);
	void initialize();
	void simulate();
	bool next_parameter_combination();
	void calculate_stats();
	void write_results();
	void help() const;
	void final_message();
};

#endif