/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __MODEL_REGISTRY_H
#define __MODEL_REGISTRY_H

#include<map>
#include "population.h"
#include "observer.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class Simulator;
class Landscape;
class ParameterReader;
class OutputWriter;

typedef map<int,Population*> population_map;
typedef vector<Observer*> observer_list;

/**
	Registry for all principal objects of the model. Registered
	objects can communicate with each other.
*/

class ModelRegistry
{
public:
	Simulator* simulator;
	ParameterReader* parameters;
	Landscape* landscape;
	population_map populations;
	observer_list observers;
	OutputWriter* writer;

	ModelRegistry();
	~ModelRegistry() {};

	void register_simulator(Simulator* _simulator);
	void register_landscape(Landscape* _landscape);
	void register_parameters(ParameterReader* _parameters);
	void register_population(Population* _population);
	void register_observer(Observer* _observer);
	void register_outputwriter(OutputWriter* _writer);

	Population* getPopulation(int _id);
	population_map& getPopulations()			{ return populations; };

	Observer* getObserver(Observer* _observer);
	observer_list* getObservers()				{ return &observers; };
	void updateCellImmigrationObserver(unsigned short _population_id, unsigned int _individual_id, cover_type _cover_id, float _xpos, float _ypos);
	void updatePatchImmigrationObserver(unsigned short _population_id, unsigned int _frompatch_id, unsigned int _topatch_id);
	void updateDistanceObserver(unsigned short _population_id, unsigned int _individual_id, bool _start, float _xpos, float _ypos);
	void updateCrossingObserver(unsigned short _population_id, cover_type _from_cover, cover_type _to_cover);
	void updateSearchTimeObserver(unsigned short _population_id, unsigned int _steps);

	void resetObservers();
};
 
#endif