/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __OUTPUTWRITER_H
#define __OUTPUTWRITER_H

#include <iostream>
#include "ibusepmexception.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class ModelRegistry;
class Observer;

/**
	OutputWriter wraps and controls all writing operations of the model,
	such as writing landscape as ascii or gif files, aggregated or raw
	results of registered observers.
*/

class OutputWriter
{
private:
	ModelRegistry*	registry;
	const char*		basename;
	ofstream*		outputfilestream;	
	ofstream*		obsrawfilestream;
	bool			write_stat_header;		//control flag for aggregated header output
	bool			write_raw_header;		//control flag for raw header output

public: 
	OutputWriter(ModelRegistry* _registry);
	~OutputWriter();

	void setBasename(const char* _basename)		{ basename = _basename; };
	const char* getBasename() const				{ return basename; };

	void report_stat_observations() throw (IbusepmException);
	void report_raw_observations() throw (IbusepmException);
	void write_asc_landscape();
	void write_gif_landscape(char* filename = NULL);

private:
	void openOutputFileStream() throw (IbusepmException);
	void openRawFileStream(Observer* _observer) throw (IbusepmException);
};

#endif