/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __PATCH_H
#define __PATCH_H

#include "defs.h"
#include "patchisovar.h"
#include "ibusepmexception.h"
#include <list>
#include <iostream>
#include <map>

#if !defined (STLPORT) || defined(__STL_USE_NAMESPACES)
using namespace std;
#endif

typedef list<PatchIsoVar*> isovar_list;
typedef map<unsigned short, unsigned int> pop_individuals;

/**
	A patch object represents a cluster of contagious cells in the
	landscape.
*/

class Patch 
{

private:
	unsigned int	patch_id;		//patch id
	cover_type		cover_id;		//landcover type (0..255)
	unsigned short	xref,yref;		//ref coordinates for most upper and most left cel of the patch
									//used as starting point for patch related iterations to save time
	unsigned short	xmax,ymax;		//used to assign population to patch more efficiently
	unsigned int	size;			//size=number of cells
	unsigned int	perimeter;		//=number of border cells (dep. on diagonal flag!)
	pop_individuals	individuals;	//counter of actual current individuals on this patch for each population

	isovar_list	isovars;			//list for patch isolation metrics

public:

	Patch();
	Patch(long _patch_id, cover_type _cover_id); 
	Patch(unsigned short _xref, unsigned short _yref, long _patch_id, cover_type _cover_id);
	~Patch() {};
	void reset();

	bool operator==(const Patch& _patch) const;
	bool operator!=(const Patch& _patch) const;

	//property methods - getter/setter

    cover_type getCover_id() const				{ return cover_id; }
	void setCover_id(cover_type _cover_id)		{ cover_id = _cover_id; };

    unsigned int getPatch_id() const			{ return patch_id; };
	void setPatch_id(unsigned int _patch_id)	{ patch_id = _patch_id; };
	
	unsigned short getXref() const				{ return xref; };
	void setXref(unsigned short _xref)			{ xref = _xref; };

	unsigned short getYref() const				{ return yref; };
	void setYref(unsigned short _yref)			{ yref = _yref; };

	unsigned short getXmax() const				{ return xmax; };
	void setXmax(unsigned short _xmax)			{ xmax = _xmax; };

	unsigned short getYmax() const				{ return ymax; };
	void setYmax(unsigned short _ymax)			{ ymax = _ymax; };

	unsigned int getSize() const				{ return size; };
	void setSize(unsigned short _size)			{ size = _size; };

	unsigned int getPerimeter() const			{ return perimeter; };
	void setPerimeter(unsigned int _perimeter)	{ perimeter = _perimeter; };

	void register_cell(int x, int y);
	void add_iso_var(PatchIsoVar* _isovar);

	void addIndividual(unsigned short _population_id) throw (IbusepmException);		
	void removeIndividual(unsigned short _population_id) throw (IbusepmException);
	unsigned int getIndividuals(unsigned short _population_id);

	const char* report() const;
	void report(ostream& strm=cout) const;
	void report_iso_vars(ostream& strm=cout) const;
};

#endif