/*************************************************************
EDGE-1.0 is an individual-based, spatially explicit simulation
model for analysing demographic and landscape effects on
population dynamics.

Copyright (C) 2005  Lutz Tischendorf

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*************************************************************/

#ifndef __SIMULATOR_H
#define __SIMULATOR_H

#include <map>
#include "ibusepmexception.h"
#include "population.h"

#if !defined (STLPORT) || defined(_STLP_USE_NAMESPACES)
using namespace std;
#endif

class ModelRegistry;

typedef map<short, activity_list> step_activities;

/**
	The simulator executes the loop over all time steps, runs and model
	parameter values. 
*/

class Simulator
{
	unsigned short		years;
	unsigned short		steps;
	unsigned short		obs_gap;
	unsigned short		runs;
	unsigned int		combination;
	unsigned short		dot_offset;
	
	ModelRegistry*		registry;
	
public:
	Simulator(ModelRegistry* _registry);
	~Simulator() {};
	void reset();

	void setYears(unsigned short _years)			{ years = _years; };
	unsigned short getYears() const					{ return years; };

	void setSteps_per_year(unsigned short _steps)	{ steps = _steps; };
	unsigned short getSteps_per_year() const		{ return steps; };

	void setRuns(unsigned short _runs)				{ runs = _runs; };
	unsigned short getRuns() const					{ return runs; };

	void setObs_gap(unsigned short _obs_gap)		{ obs_gap = _obs_gap; };
	unsigned short getObs_gap() const				{ return obs_gap; };

	unsigned short getObs_steps();

	void simulate();
	void observe(unsigned short _run, unsigned short _step);

private:
	void reinitialize_populations();
	void activate_populations(unsigned short _step);
	void verify_status() throw (IbusepmException);
	void write_dot(int _run, int& _gap);
	void showProgress();
};

#endif